#include "control.h"

void CControl::Clear()
{
     ZeroMemory(this, sizeof(CControl));

     m_vAxis[0] = D3DXVECTOR3(1.0f, 0.0f, 0.0f);
     m_vAxis[1] = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
     m_vAxis[2] = D3DXVECTOR3(0.0f, 0.0f, 1.0f);

     D3DXMatrixIdentity(&m_mRotation);
     D3DXMatrixIdentity(&m_mTranslation);
     D3DXMatrixIdentity(&m_mTransformation);
}

void CControl::RotateX(float fAngle)
{
    Rotate(0, fAngle);
}

void CControl::RotateY(float fAngle)
{
    Rotate(1, fAngle);
}

void CControl::RotateZ(float fAngle)
{
    Rotate(2, fAngle);
}

void CControl::Rotate(int iAxis,
                      float fAngle)
{
    D3DXMATRIX  mTemp;
    D3DXVECTOR3 vTemp;

    D3DXMatrixRotationAxis(&mTemp, &m_vAxis[iAxis], fAngle);
    m_mRotation *= mTemp;
    
    for(int a = 0; a < 3; a++)
    {
        vTemp = m_vAxis[a];
        D3DXVec3TransformCoord(&m_vAxis[a], &vTemp, &mTemp);
        D3DXVec3Normalize(&m_vAxis[a], &m_vAxis[a]);
    }    
}

void CControl::TranslateX(float fX)
{
    Translate(D3DXVECTOR3(fX, 0.0f, 0.0f));
}

void CControl::TranslateY(float fY)
{
    Translate(D3DXVECTOR3(0.0f, fY, 0.0f));
}

void CControl::TranslateZ(float fZ)
{
    Translate(D3DXVECTOR3(0.0f, 0.0f, fZ));
}

void CControl::Translate(D3DXVECTOR3 vTranslation)
{
    m_vPosition += D3DXVECTOR3(vTranslation.x * m_vAxis[0].x + vTranslation.y * m_vAxis[1].x + vTranslation.z * m_vAxis[2].x,
                               vTranslation.x * m_vAxis[1].y + vTranslation.y * m_vAxis[1].y + vTranslation.z * m_vAxis[2].y,
                               vTranslation.x * m_vAxis[2].z + vTranslation.y * m_vAxis[1].z + vTranslation.z * m_vAxis[2].z);

    D3DXMatrixTranslation(&m_mTranslation,
                          m_vPosition.x, m_vPosition.y, m_vPosition.z);
}

void CControl::SetPosition(D3DXVECTOR3 vNewPosition)
{
    m_vPosition = vNewPosition;

    D3DXMatrixTranslation(&m_mTranslation,
                          m_vPosition.x, m_vPosition.y, m_vPosition.z);
}

D3DXVECTOR3 CControl::GetRelativePosition(D3DXVECTOR3 vRelativePosition)
{
    return D3DXVECTOR3(m_vPosition.x + (vRelativePosition.x * m_vAxis[0].x) + (vRelativePosition.y * m_vAxis[1].x) + (vRelativePosition.z * m_vAxis[2].x),
                       m_vPosition.y + (vRelativePosition.y * m_vAxis[1].y) + (vRelativePosition.z * m_vAxis[2].y) + (vRelativePosition.x * m_vAxis[0].y),
                       m_vPosition.z + (vRelativePosition.z * m_vAxis[2].z) + (vRelativePosition.x * m_vAxis[0].z) + (vRelativePosition.y * m_vAxis[1].z));
}

D3DXVECTOR3 CControl::GetRelativePositionAxis(D3DXVECTOR3 vTarget)
{
    D3DXVECTOR3 vTemp;
    D3DXMATRIX  mLookAt;

    D3DXMatrixLookAtLH(&mLookAt, &m_vPosition, &D3DXVECTOR3(m_vPosition + m_vAxis[2]), &m_vAxis[1]);
    D3DXVec3TransformCoord(&vTemp, &vTarget, &mLookAt);

    return vTemp;
}

D3DXMATRIX CControl::GetRelativePositionMatrix(D3DXVECTOR3 vRelativePosition,
                                               bool bUseRotation)
{
    D3DXMATRIX mTemp;

    vRelativePosition = GetRelativePosition(vRelativePosition);

    D3DXMatrixTranslation(&mTemp,
                          vRelativePosition.x, vRelativePosition.y, vRelativePosition.z);

    if(bUseRotation)
    {
        return m_mRotation * mTemp;
    }

    return mTemp;
}

CControl CControl::GetRelativePositionControl(D3DXVECTOR3 vRelativePosition)
{
    CControl Temp;

    memcpy(&Temp, this, sizeof(CControl));
    Temp.SetPosition(GetRelativePosition(vRelativePosition));
    Temp.UpdateTransformationMatrix();

    return Temp;
}

void CControl::SetAxes(D3DXVECTOR3 vAxisX,
                       D3DXVECTOR3 vAxisY,
                       D3DXVECTOR3 vAxisZ)
{
    m_vAxis[0] = vAxisX;
    m_vAxis[1] = vAxisY;
    m_vAxis[2] = vAxisZ;

    D3DXMatrixLookAtLH(&m_mRotation, &D3DXVECTOR3(0.0f, 0.0f, 0.0f),
                                     &vAxisZ,
                                     &vAxisY);
    UpdateTransformationMatrix();
}

void CControl::SetDirection(D3DXVECTOR3 vDirection,
                            float fRotation)
{
    D3DXMATRIX mRotation;

    m_vAxis[0] = D3DXVECTOR3(1.0f, 0.0f, 0.0f);
    m_vAxis[1] = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
    m_vAxis[2] = vDirection;

    D3DXMatrixRotationAxis(&mRotation, &vDirection, fRotation);
    D3DXVec3TransformCoord(&m_vAxis[0], &m_vAxis[0], &mRotation);
    D3DXVec3TransformCoord(&m_vAxis[1], &m_vAxis[1], &mRotation);

    SetAxes(m_vAxis[0], m_vAxis[1], m_vAxis[2]);

    UpdateTransformationMatrix();
}

void CControl::UpdateTransformationMatrix()
{
    D3DXMatrixTranslation(&m_mTranslation,
                          m_vPosition.x, m_vPosition.y, m_vPosition.z);

    m_mTransformation = m_mRotation * m_mTranslation;
}