#include "mesh.h"

HRESULT MeshCreate(CMesh **ppMesh,
                   LPSTR psFilename,
                   LPDIRECT3DDEVICE8 pD3DDevice,                
                   BOOL bManageMaterials,
                   BOOL bEqualMaterials,
                   BOOL bEqualTextures)
{
    CMesh           *pTemp;
    LPD3DXBUFFER    pMaterialBuffer;
    LPD3DXBUFFER    pAdjacencyBuffer;
    DWORD           dwNumMaterials;
    D3DXMATERIAL    *pMaterial;
    HRESULT         r;

    *ppMesh = NULL;

    pTemp = (CMesh*)(calloc(1, sizeof(CMesh)));
    pTemp->m_pD3DDevice         = pD3DDevice;
    pTemp->m_bManageMaterials   = bManageMaterials;
    pTemp->m_bEqualMaterials    = bEqualMaterials;
    pTemp->m_bEqualTextures     = bEqualTextures;

    r = D3DXLoadMeshFromX(psFilename,
                          0,
                          pD3DDevice,
                          &pAdjacencyBuffer,
                          &pMaterialBuffer,
                          &dwNumMaterials,
                          &pTemp->m_pMesh);
    if(FAILED(r)) return r;

    pTemp->m_dwNumMaterials = dwNumMaterials;

    pMaterial = (D3DXMATERIAL*)pMaterialBuffer->GetBufferPointer();

    for(int a = 0; a < (int)(dwNumMaterials); a++)
    {
        pTemp->m_Material[a] = pMaterial[a].MatD3D;
        pTemp->m_Material[a].Ambient.r = pMaterial[a].MatD3D.Diffuse.r / 2.0f;
        pTemp->m_Material[a].Ambient.g = pMaterial[a].MatD3D.Diffuse.g / 2.0f;
        pTemp->m_Material[a].Ambient.b = pMaterial[a].MatD3D.Diffuse.b / 2.0f;
    }

    pMaterialBuffer->Release();
    pAdjacencyBuffer->Release();

    *ppMesh = pTemp;

    return S_OK;
}

void CMesh::Release()
{
    for(int a = 0; a < (int)(m_dwNumMaterials); a++)
    {
        if(m_pTexture[a] != NULL) m_pTexture[a]->Release();
    }
    m_pMesh->Release();

    free(this);
}

HRESULT CMesh::Render(BOOL bEnableOptimization)
{
    HRESULT r;

    if(m_bManageMaterials)
    {
        if(m_bEqualMaterials)
        {
            if((bEnableOptimization && g_pmLastMeshMaterial != &m_Material[0]) ||
               !bEnableOptimization)
            {
                m_pD3DDevice->SetMaterial(&m_Material[0]);
                if(bEnableOptimization) g_pmLastMeshMaterial = &m_Material[0];
            }
        }
        if(m_bEqualTextures)
        {
            if((bEnableOptimization && g_pLastMeshTexture != m_pTexture[0]) ||
               !bEnableOptimization)
            {
                m_pD3DDevice->SetTexture(0, m_pTexture[0]);
                if(bEnableOptimization) g_pLastMeshTexture = m_pTexture[0];
            }
        }
    }

    for(int a = 0; a < (int)(m_dwNumMaterials); a++)
    {
        if(m_bManageMaterials)
        {
            if(!m_bEqualMaterials)
            {
                if((bEnableOptimization && g_pmLastMeshMaterial != &m_Material[a]) ||
                   !bEnableOptimization)
                {
                    m_pD3DDevice->SetMaterial(&m_Material[a]);
                    if(bEnableOptimization) g_pmLastMeshMaterial = &m_Material[a];
                }
            }
            if(!m_bEqualTextures)
            {
                if((bEnableOptimization && g_pLastMeshTexture != m_pTexture[a]) ||
                   !bEnableOptimization)
                {
                    m_pD3DDevice->SetTexture(0, m_pTexture[a]);
                    if(bEnableOptimization) g_pLastMeshTexture = m_pTexture[a];
                }
            }
        }

        r = m_pMesh->DrawSubset(a);
        if(FAILED(r)) return r;
    }

    return S_OK;
}