#include "utils.h"

float FloatRandom(float fMinimum,
                  float fMaximum)
{
    float   fDifference;
    long    lTemp;

    fDifference = fMaximum - fMinimum;
    lTemp = rand() % 1001;

    return fMinimum + ((float)(lTemp / 1000.0f) * fDifference);
}

D3DXMATRIX SetCamera(D3DXVECTOR3 vPosition,
                     D3DXVECTOR3 vLookAt,
                     D3DXVECTOR3 vUp,
                     LPDIRECT3DDEVICE8 pD3DDevice)
{
    D3DXMATRIX mCameraMatrix;

    D3DXMatrixLookAtLH(&mCameraMatrix, &vPosition,
                                       &vLookAt,
                                       &vUp);

    pD3DDevice->SetTransform(D3DTS_VIEW, &mCameraMatrix);

    return mCameraMatrix;
}

void SetWorldTranslation(D3DXVECTOR3 vTranslation,
                         LPDIRECT3DDEVICE8 pD3DDevice)
{
    D3DXMATRIX mTranslation;

    D3DXMatrixTranslation(&mTranslation, vTranslation.x, vTranslation.y, vTranslation.z);

    pD3DDevice->SetTransform(D3DTS_WORLD, (D3DMATRIX*)(&mTranslation));
}

D3DCOLORVALUE D3DRGB(float r,
                     float g,
                     float b)
{
    D3DCOLORVALUE cTemp;

    cTemp.r = r;
    cTemp.g = g;
    cTemp.b = b;
    cTemp.a = 1.0f;

    return cTemp;
}

DWORD FloatToDWord(float fValue)
{
    return *((DWORD*)(&fValue));
}

HRESULT SetListener(LPDIRECTSOUND3DLISTENER8 pListener,
                    D3DXVECTOR3 vPosition,
                    D3DXVECTOR3 vVelocity,
                    D3DXVECTOR3 vAxisY,
                    D3DXVECTOR3 vAxisZ,
                    float fDistanceFactor,
                    float fRolloffFactor,
                    float fDopplerFactor)
{
    DS3DLISTENER    Listener;
    HRESULT         r;

    Listener.dwSize = sizeof(Listener);
    r = pListener->GetAllParameters(&Listener);
    if(FAILED(r)) return r;

    Listener.vPosition          = vPosition;
    Listener.vVelocity          = vVelocity;
    Listener.vOrientTop         = vAxisY;
    Listener.vOrientFront       = vAxisZ;
    Listener.flDistanceFactor   = fDistanceFactor;
    Listener.flRolloffFactor    = fRolloffFactor;
    Listener.flDopplerFactor    = fDopplerFactor;

    return pListener->SetAllParameters(&Listener, DS3D_IMMEDIATE);
}

HRESULT Set3DBuffer(LPDIRECTSOUND3DBUFFER8 pBuffer,
                    D3DXVECTOR3 vPosition,
                    D3DXVECTOR3 vVelocity,
                    float fMinDistance,
                    float fMaxDistance)
{
    DS3DBUFFER  Buffer;
    HRESULT     r;

    Buffer.dwSize = sizeof(Buffer);
    r = pBuffer->GetAllParameters(&Buffer);
    if(FAILED(r)) return r;

    Buffer.dwSize           = sizeof(Buffer);
    Buffer.vPosition        = vPosition;
    Buffer.vVelocity        = vVelocity;
    Buffer.flMinDistance    = fMinDistance;
    Buffer.flMaxDistance    = fMaxDistance;

    return pBuffer->SetAllParameters(&Buffer, DS3D_IMMEDIATE);
}

HRESULT RenderPointSprite(LPDIRECT3DDEVICE8 pD3DDevice,
                          LPDIRECT3DTEXTURE8 pTexture,
                          D3DXVECTOR3 vPosition,
                          D3DCOLOR cColor)
{
    HRESULT             r;
    POINT_SPRITE_VERTEX vVertex;

    vVertex.v = vPosition;
    vVertex.c = cColor;

    if(g_pLastMeshTexture != pTexture)
    {
        pD3DDevice->SetVertexShader(POINT_SPRITE_VERTEX_FVF);
        pD3DDevice->SetTexture(0, pTexture);
        g_pLastMeshTexture = pTexture;
    }

    r = pD3DDevice->DrawPrimitiveUP(D3DPT_POINTLIST, 1, &vVertex, sizeof(POINT_SPRITE_VERTEX));
    if(FAILED(r)) return r;

    return S_OK;
}

void Delay(DWORD dwDuration)
{
    DWORD dwStartTime;

    dwStartTime = timeGetTime();
    while(timeGetTime() <= dwStartTime + dwDuration) {}
}